from datetime import datetime
from typing import Any, Dict, Optional

from loguru import logger

from .models import VersionType


class VersionFormatter:
    """Maneja diferentes formatos de versiones basados en la configuración."""

    def __init__(self, config: Optional[Dict[str, Any]] = None):
        self.config = config or {}
        self.format_type = self.config.get("version_format", "semver")

    def bump_version(self, current_version: str, version_type: VersionType) -> str:
        """
        Incrementa la versión según el formato configurado.
        """
        logger.debug(f"self.format_type: {self.format_type}")
        if self.format_type == "semver":
            return self._bump_semver(current_version, version_type)
        elif self.format_type == "date":
            return self._bump_date(current_version, version_type)
        else:
            raise ValueError(f"Formato de versión no soportado: {self.format_type}")

    def _bump_semver(self, current_version: str, version_type: VersionType) -> str:
        """Incrementa versión en formato semántico (MAJOR.MINOR.PATCH)."""
        major, minor, patch = map(int, current_version.split("."))

        if version_type == VersionType.MAJOR:
            return f"{major + 1}.0.0"
        elif version_type == VersionType.MINOR:
            return f"{major}.{minor + 1}.0"
        else:  # PATCH
            return f"{major}.{minor}.{patch + 1}"

    def _bump_date(self, current_version: str, version_type: VersionType) -> str:
        """
        Incrementa versión en formato de fecha (YYYY.MM.DD.BUILD).
        """
        if version_type == VersionType.PATCH:
            raise ValueError("PATCH no soportada para formato date")

        date_format = self.config.get("date_format", "%Y.%m.%d")
        current_date = datetime.now().strftime(date_format)

        if version_type == VersionType.MINOR:
            if "." not in current_version:
                build = 1
            else:
                *_, build_str = current_version.split(".")
                try:
                    build = int(build_str) + 1
                except ValueError:
                    build = 1
        else:
            build = 1

        return f"{current_date}.{build}"
